@@ -12,10 +12,10 @@
 #define SUITESPARSE_GPURUNTIME_HPP
 
 // Version information:
-#define SUITESPARSE_GPURUNTIME_DATE "Dec 9, 2022"
-#define SUITESPARSE_GPURUNTIME_MAIN_VERSION   2
-#define SUITESPARSE_GPURUNTIME_SUB_VERSION    0
-#define SUITESPARSE_GPURUNTIME_SUBSUB_VERSION 2
+#define SUITESPARSE_GPURUNTIME_DATE "Sept 8, 2023"
+#define SUITESPARSE_GPURUNTIME_MAIN_VERSION   3
+#define SUITESPARSE_GPURUNTIME_SUB_VERSION    2
+#define SUITESPARSE_GPURUNTIME_SUBSUB_VERSION 0
 
 #define SUITESPARSE_GPURUNTIME_VER_CODE(main,sub) ((main) * 1000 + (sub))
 #define SUITESPARSE_GPURUNTIME_VERSION \
