@@ -814,10 +814,10 @@ void *klu_l_realloc (size_t, size_t, size_t, void *, klu_l_common *) ;
  *      #endif
  */
 
-#define KLU_DATE "Jan 17, 2023"
+#define KLU_DATE "Sept 8, 2023"
 #define KLU_MAIN_VERSION   2
-#define KLU_SUB_VERSION    0
-#define KLU_SUBSUB_VERSION 3
+#define KLU_SUB_VERSION    2
+#define KLU_SUBSUB_VERSION 0
 
 #define KLU_VERSION_CODE(main,sub) ((main) * 1000 + (sub))
 #define KLU_VERSION KLU_VERSION_CODE(KLU_MAIN_VERSION,KLU_SUB_VERSION)
